@@ -1,5 +1,5 @@
 /*
- * $Id: _psutil_bsd.c 502 2010-02-21 12:33:41Z billiejoex@gmail.com $
+ * $Id: _psutil_bsd.c 944 2011-03-04 17:05:24Z g.rodola $
  *
  * FreeBSD platform-specific module methods for _psutil_bsd
  */
@@ -12,129 +12,56 @@
 #include <signal.h>
 #include <sys/types.h>
 #include <sys/sysctl.h>
+#include <sys/param.h>
 #include <sys/user.h>
 #include <sys/proc.h>
 #include <sys/vmmeter.h>  /* needed for vmtotal struct */
 
 #include "_psutil_bsd.h"
+#include "_psutil_common.h"
 #include "arch/bsd/process_info.h"
 
 
-static PyObject *NoSuchProcessException;
+// convert a timeval struct to a double
+#define TV2DOUBLE(t)    ((t).tv_sec + (t).tv_usec / 1000000.0)
+
 
 /*
- * define the psutil C module methods and initialize the module.
+ * Utility function which fills a kinfo_proc struct based on process pid
  */
-static PyMethodDef PsutilMethods[] =
+static int
+get_kinfo_proc(const pid_t pid, struct kinfo_proc *proc)
 {
-     {"get_pid_list", get_pid_list, METH_VARARGS,
-     	"Returns a list of PIDs currently running on the system"},
-     {"get_process_info", get_process_info, METH_VARARGS,
-        "Return a tuple containing a set of information about the "
-        "process (pid, ppid, name, path, cmdline)"},
-     {"get_cpu_times", get_cpu_times, METH_VARARGS,
-       	"Return tuple of user/kern time for the given PID"},
-     {"get_num_cpus", get_num_cpus, METH_VARARGS,
-       	"Return number of CPUs on the system"},
-     {"get_process_create_time", get_process_create_time, METH_VARARGS,
-         "Return a float indicating the process create time expressed in "
-         "seconds since the epoch"},
-     {"get_memory_info", get_memory_info, METH_VARARGS,
-         "Return a tuple of RSS/VMS memory information"},
-     {"get_total_phymem", get_total_phymem, METH_VARARGS,
-         "Return the total amount of physical memory, in bytes"},
-     {"get_avail_phymem", get_avail_phymem, METH_VARARGS,
-         "Return the amount of available physical memory, in bytes"},
-     {"get_total_virtmem", get_total_virtmem, METH_VARARGS,
-         "Return the total amount of virtual memory, in bytes"},
-     {"get_avail_virtmem", get_avail_virtmem, METH_VARARGS,
-         "Return the amount of available virtual memory, in bytes"},
-     {"get_system_cpu_times", get_system_cpu_times, METH_VARARGS,
-         "Return system cpu times as a tuple (user, system, nice, idle, irc)"},
-     {NULL, NULL, 0, NULL}
-};
+    int mib[4];
+    size_t size;
+    mib[0] = CTL_KERN;
+    mib[1] = KERN_PROC;
+    mib[2] = KERN_PROC_PID;
+    mib[3] = pid;
 
+    size = sizeof(struct kinfo_proc);
 
-struct module_state {
-    PyObject *error;
-};
-
-#if PY_MAJOR_VERSION >= 3
-#define GETSTATE(m) ((struct module_state*)PyModule_GetState(m))
-#else
-#define GETSTATE(m) (&_state)
-static struct module_state _state;
-#endif
-
-#if PY_MAJOR_VERSION >= 3
-
-static int psutil_bsd_traverse(PyObject *m, visitproc visit, void *arg) {
-    Py_VISIT(GETSTATE(m)->error);
-    return 0;
-}
-
-static int psutil_bsd_clear(PyObject *m) {
-    Py_CLEAR(GETSTATE(m)->error);
-    return 0;
-}
-
-
-static struct PyModuleDef moduledef = {
-        PyModuleDef_HEAD_INIT,
-        "psutil_bsd",
-        NULL,
-        sizeof(struct module_state),
-        PsutilMethods,
-        NULL,
-        psutil_bsd_traverse,
-        psutil_bsd_clear,
-        NULL
-};
-
-#define INITERROR return NULL
-
-PyObject *
-PyInit__psutil_bsd(void)
-
-#else
-#define INITERROR return
-
-void
-init_psutil_bsd(void)
-#endif
-{
-#if PY_MAJOR_VERSION >= 3
-    PyObject *module = PyModule_Create(&moduledef);
-#else
-    PyObject *module = Py_InitModule("_psutil_bsd", PsutilMethods);
-#endif
-    if (module == NULL)
-        INITERROR;
-    struct module_state *st = GETSTATE(module);
-
-    st->error = PyErr_NewException("_psutil_bsd.Error", NULL, NULL);
-    if (st->error == NULL) {
-        Py_DECREF(module);
-        INITERROR;
+    if (sysctl((int*)mib, 4, proc, &size, NULL, 0) == -1) {
+        PyErr_SetFromErrno(PyExc_OSError);
+        return -1;
     }
 
-    NoSuchProcessException = PyErr_NewException("_psutil_bsd.NoSuchProcess",
-                                                 NULL, NULL);
-    Py_INCREF(NoSuchProcessException);
-    PyModule_AddObject(module, "NoSuchProcess", NoSuchProcessException);
-
-#if PY_MAJOR_VERSION >= 3
-    return module;
-#endif
+    /*
+     * sysctl stores 0 in the size if we can't find the process information.
+     */
+    if (size == 0) {
+        NoSuchProcess();
+        return -1;
+    }
+    return 0;
 }
 
 
-
-
 /*
  * Return a Python list of all the PIDs running on the system.
  */
-static PyObject* get_pid_list(PyObject* self, PyObject* args)
+static PyObject*
+get_pid_list(PyObject* self, PyObject* args)
 {
     kinfo_proc *proclist = NULL;
     kinfo_proc *orig_address = NULL;
@@ -165,144 +92,305 @@ static PyObject* get_pid_list(PyObject* self, PyObject* args)
 
 
 /*
- * Return a Python tuple containing a set of information about the process:
- * (pid, ppid, name, path, cmdline).
+ * Return a Python float indicating the system boot time expressed in
+ * seconds since the epoch.
  */
-static PyObject* get_process_info(PyObject* self, PyObject* args)
+static PyObject*
+get_system_boot_time(PyObject* self, PyObject* args)
 {
+    /* fetch sysctl "kern.boottime" */
+    static int request[2] = { CTL_KERN, KERN_BOOTTIME };
+    struct timeval result;
+    size_t result_len = sizeof result;
+    time_t boot_time = 0;
 
-    int mib[4];
-    size_t len;
-    struct kinfo_proc kp;
-	long pid;
-    PyObject* infoTuple = NULL;
-    PyObject* arglist = NULL;
-
-	// the argument passed should be a process id
-	if (! PyArg_ParseTuple(args, "l", &pid)) {
-		return PyErr_Format(PyExc_RuntimeError, "Invalid argument - no PID provided.");
-	}
-
-    if (0 == pid) {
-        // USER   PID %CPU %MEM   VSZ   RSS  TT  STAT STARTED      TIME COMMAND
-        // root     0  0.0  0.0     0     0  ??  DLs  12:22AM   0:00.13 [swapper]
-        return Py_BuildValue("llssNll", pid, 0, "swapper", "", Py_BuildValue("[]"), 0, 0);
+    if (sysctl(request, 2, &result, &result_len, NULL, 0) == -1) {
+        PyErr_SetFromErrno(0);
+        return NULL;
     }
-
-    // build the mib to pass to sysctl to tell it what PID and what info we want
-    len = 4;
-    sysctlnametomib("kern.proc.pid", mib, &len);
-    mib[0] = CTL_KERN;
-    mib[1] = KERN_PROC;
-    mib[2] = KERN_PROC_PID;
-    mib[3] = pid;
-
-    // fetch the info with sysctl()
-    len = sizeof(kp);
-    if (sysctl(mib, 4, &kp, &len, NULL, 0) == -1) {
-        // raise an exception if it failed, since we won't get any data
-        if (ESRCH == errno) {
-            return PyErr_Format(NoSuchProcessException, "No process found with pid %lu", pid);
-        }
-        return PyErr_SetFromErrno(PyExc_OSError);
-    }
-
-    if (len > 0) { // if 0 then no data was retrieved
-
-        // get the commandline, since we got everything else
-        arglist = get_arg_list(pid);
-
-        // get_arg_list() returns NULL only if getcmdargs failed with ESRCH
-        // (no process with that PID)
-        if (NULL == arglist) {
-            return PyErr_Format(NoSuchProcessException,
-                                "No such process found with pid %lu", pid);
-        }
-
-        // hooray, we got all the data, so return it as a tuple to be passed to
-        // ProcessInfo() constructor
-        infoTuple = Py_BuildValue("llssNll", pid, (long)kp.ki_ppid, kp.ki_comm, "",
-                                  arglist, (long)kp.ki_ruid, (long)kp.ki_rgid);
-
-        if (NULL == infoTuple) {
-            PyErr_SetString(PyExc_RuntimeError,
-                            "Failed to build process information tuple!");
-        }
-        return infoTuple;
-    }
-
-    // something went wrong, throw an error
-	return PyErr_Format(PyExc_RuntimeError,
-                        "Failed to retrieve process information.");
+    boot_time = result.tv_sec;
+    return Py_BuildValue("f", (float)boot_time);
 }
 
-// convert a timeval struct to a double
-#define TV2DOUBLE(t)    ((t).tv_sec + (t).tv_usec / 1000000.0)
+
+/*
+ * Return process name from kinfo_proc as a Python string.
+ */
+static PyObject*
+get_process_name(PyObject* self, PyObject* args)
+{
+    long pid;
+    struct kinfo_proc kp;
+    if (! PyArg_ParseTuple(args, "l", &pid)) {
+        return NULL;
+    }
+    if (get_kinfo_proc(pid, &kp) == -1) {
+        return NULL;
+    }
+    return Py_BuildValue("s", kp.ki_comm);
+}
+
+
+/*
+ * Return process pathname executable.
+ * Thanks to Robert N. M. Watson:
+ * http://fxr.googlebit.com/source/usr.bin/procstat/procstat_bin.c?v=8-CURRENT
+ */
+static PyObject*
+get_process_exe(PyObject* self, PyObject* args)
+{
+    long pid;
+    char pathname[PATH_MAX];
+    int error;
+    int mib[4];
+    size_t size;
+
+    if (! PyArg_ParseTuple(args, "l", &pid)) {
+        return NULL;
+    }
+
+    mib[0] = CTL_KERN;
+    mib[1] = KERN_PROC;
+    mib[2] = KERN_PROC_PATHNAME;
+    mib[3] = pid;
+
+    size = sizeof(pathname);
+    error = sysctl(mib, 4, pathname, &size, NULL, 0);
+    if (error == -1) {
+        PyErr_SetFromErrno(PyExc_OSError);
+        return NULL;
+    }
+    if (size == 0 || strlen(pathname) == 0) {
+        if (pid_exists(pid) == 0) {
+            return NoSuchProcess();
+        }
+        else {
+            strcpy(pathname, "");
+        }
+    }
+    return Py_BuildValue("s", pathname);
+}
+
+
+/*
+ * Return process cmdline as a Python list of cmdline arguments.
+ */
+static PyObject*
+get_process_cmdline(PyObject* self, PyObject* args)
+{
+    long pid;
+    PyObject* arglist = NULL;
+
+    if (! PyArg_ParseTuple(args, "l", &pid)) {
+        return NULL;
+    }
+
+    // get the commandline, defined in arch/bsd/process_info.c
+    arglist = get_arg_list(pid);
+
+    // get_arg_list() returns NULL only if getcmdargs failed with ESRCH
+    // (no process with that PID)
+    if (NULL == arglist) {
+        return PyErr_SetFromErrno(PyExc_OSError);
+    }
+    return Py_BuildValue("N", arglist);
+}
+
+
+/*
+ * Return process parent pid from kinfo_proc as a Python integer.
+ */
+static PyObject*
+get_process_ppid(PyObject* self, PyObject* args)
+{
+    long pid;
+    struct kinfo_proc kp;
+    if (! PyArg_ParseTuple(args, "l", &pid)) {
+        return NULL;
+    }
+    if (get_kinfo_proc(pid, &kp) == -1) {
+        return NULL;
+    }
+    return Py_BuildValue("l", (long)kp.ki_ppid);
+}
+
+
+/*
+ * Return process status as a Python integer.
+ */
+static PyObject*
+get_process_status(PyObject* self, PyObject* args)
+{
+    long pid;
+    struct kinfo_proc kp;
+    if (! PyArg_ParseTuple(args, "l", &pid)) {
+        return NULL;
+    }
+    if (get_kinfo_proc(pid, &kp) == -1) {
+        return NULL;
+    }
+    return Py_BuildValue("i", (int)kp.ki_stat);
+}
+
+
+/*
+ * Return process real, effective and saved user ids from kinfo_proc
+ * as a Python tuple.
+ */
+static PyObject*
+get_process_uids(PyObject* self, PyObject* args)
+{
+    long pid;
+    struct kinfo_proc kp;
+    if (! PyArg_ParseTuple(args, "l", &pid)) {
+        return NULL;
+    }
+    if (get_kinfo_proc(pid, &kp) == -1) {
+        return NULL;
+    }
+    return Py_BuildValue("lll", (long)kp.ki_ruid,
+                                (long)kp.ki_uid,
+                                (long)kp.ki_svuid);
+}
+
+
+/*
+ * Return process real, effective and saved group ids from kinfo_proc
+ * as a Python tuple.
+ */
+static PyObject*
+get_process_gids(PyObject* self, PyObject* args)
+{
+    long pid;
+    struct kinfo_proc kp;
+    if (! PyArg_ParseTuple(args, "l", &pid)) {
+        return NULL;
+    }
+    if (get_kinfo_proc(pid, &kp) == -1) {
+        return NULL;
+    }
+    return Py_BuildValue("lll", (long)kp.ki_rgid,
+                                (long)kp.ki_groups[0],
+                                (long)kp.ki_svuid);
+}
+
+
+/*
+ * Return number of threads used by process as a Python integer.
+ */
+static PyObject*
+get_process_num_threads(PyObject* self, PyObject* args)
+{
+    long pid;
+    struct kinfo_proc kp;
+    if (! PyArg_ParseTuple(args, "l", &pid)) {
+        return NULL;
+    }
+    if (get_kinfo_proc(pid, &kp) == -1) {
+        return NULL;
+    }
+    return Py_BuildValue("l", (long)kp.ki_numthreads);
+}
+
+
+/*
+ * Retrieves all threads used by process returning a list of tuples
+ * including thread id, user time and system time.
+ * Thanks to Robert N. M. Watson:
+ * http://fxr.googlebit.com/source/usr.bin/procstat/procstat_threads.c?v=8-CURRENT
+ */
+static PyObject*
+get_process_threads(PyObject* self, PyObject* args)
+{
+    long pid;
+    int mib[4];
+    struct kinfo_proc *kip;
+    struct kinfo_proc *kipp;
+    int error;
+    unsigned int i;
+    size_t size;
+    PyObject* retList = PyList_New(0);
+    PyObject* pyTuple = NULL;
+
+    if (! PyArg_ParseTuple(args, "l", &pid)) {
+        return NULL;
+    }
+
+    /*
+     * We need to re-query for thread information, so don't use *kipp.
+     */
+    mib[0] = CTL_KERN;
+    mib[1] = KERN_PROC;
+    mib[2] = KERN_PROC_PID | KERN_PROC_INC_THREAD;
+    mib[3] = pid;
+
+    size = 0;
+    error = sysctl(mib, 4, NULL, &size, NULL, 0);
+    if (error == -1) {
+        PyErr_SetFromErrno(PyExc_OSError);
+        return NULL;
+	}
+    if (size == 0) {
+        return NoSuchProcess();
+    }
+
+    kip = malloc(size);
+    if (kip == NULL) {
+        PyErr_SetFromErrno(PyExc_OSError);
+        return NULL;
+    }
+
+    error = sysctl(mib, 4, kip, &size, NULL, 0);
+    if (error == -1) {
+        PyErr_SetFromErrno(PyExc_OSError);
+        return NULL;
+    }
+    if (size == 0) {
+        return NoSuchProcess();
+    }
+
+    for (i = 0; i < size / sizeof(*kipp); i++) {
+        kipp = &kip[i];
+        pyTuple = Py_BuildValue("Idd", kipp->ki_tid,
+                                       TV2DOUBLE(kipp->ki_rusage.ru_utime),
+                                       TV2DOUBLE(kipp->ki_rusage.ru_stime)
+                                );
+        PyList_Append(retList, pyTuple);
+        Py_XDECREF(pyTuple);
+    }
+    free(kip);
+    return retList;
+}
 
 
 /*
  * Return a Python tuple (user_time, kernel_time)
  */
-static PyObject* get_cpu_times(PyObject* self, PyObject* args)
+static PyObject*
+get_cpu_times(PyObject* self, PyObject* args)
 {
-
-    int mib[4];
-    size_t len;
-    struct kinfo_proc kp;
-	long pid;
+    long pid;
     double user_t, sys_t;
-    PyObject* timeTuple = NULL;
-
-	// the argument passed should be a process id
-	if (! PyArg_ParseTuple(args, "l", &pid)) {
-		return PyErr_Format(PyExc_RuntimeError,
-                            "Invalid argument - no PID provided.");
-	}
-
-    // build the mib to pass to sysctl to tell it what PID and what info we want
-    len = 4;
-    sysctlnametomib("kern.proc.pid", mib, &len);
-    mib[0] = CTL_KERN;
-    mib[1] = KERN_PROC;
-    mib[2] = KERN_PROC_PID;
-    mib[3] = pid;
-
-    // fetch the info with sysctl()
-    len = sizeof(kp);
-    if (sysctl(mib, 4, &kp, &len, NULL, 0) == -1) {
-        // raise an exception if it failed, since we won't get any data
-        if (ESRCH == errno) {
-            return PyErr_Format(NoSuchProcessException,
-                                "No process found with pid %lu", pid);
-        }
-        return PyErr_SetFromErrno(PyExc_OSError);
+    struct kinfo_proc kp;
+    if (! PyArg_ParseTuple(args, "l", &pid)) {
+        return NULL;
     }
-
-    if (len > 0) { // if 0 then no data was retrieved
-        user_t = TV2DOUBLE(kp.ki_rusage.ru_utime);
-        sys_t = TV2DOUBLE(kp.ki_rusage.ru_stime);
-
-        // convert from microseconds to seconds
-        timeTuple = Py_BuildValue("(dd)", user_t, sys_t);
-        if (NULL == timeTuple) {
-            PyErr_SetString(PyExc_RuntimeError,
-                            "Failed to build process CPU times tuple!");
-        }
-        return timeTuple;
+    if (get_kinfo_proc(pid, &kp) == -1) {
+        return NULL;
     }
-
-    // something went wrong, throw an error
-	return PyErr_Format(PyExc_RuntimeError,
-                        "Failed to retrieve process CPU times.");
+    // convert from microseconds to seconds
+    user_t = TV2DOUBLE(kp.ki_rusage.ru_utime);
+    sys_t = TV2DOUBLE(kp.ki_rusage.ru_stime);
+    return Py_BuildValue("(dd)", user_t, sys_t);
 }
 
 
 /*
  * Return a Python integer indicating the number of CPUs on the system
  */
-static PyObject* get_num_cpus(PyObject* self, PyObject* args)
+static PyObject*
+get_num_cpus(PyObject* self, PyObject* args)
 {
-
     int mib[2];
     int ncpu;
     size_t len;
@@ -324,68 +412,59 @@ static PyObject* get_num_cpus(PyObject* self, PyObject* args)
  * Return a Python float indicating the process create time expressed in
  * seconds since the epoch.
  */
-static PyObject* get_process_create_time(PyObject* self, PyObject* args)
+static PyObject*
+get_process_create_time(PyObject* self, PyObject* args)
 {
-    int mib[4];
     long pid;
-    size_t len;
     struct kinfo_proc kp;
-
-	// the argument passed should be a process id
-	if (! PyArg_ParseTuple(args, "l", &pid)) {
-		return PyErr_Format(PyExc_RuntimeError,
-                            "Invalid argument - no PID provided.");
-	}
-
-    len = 4;
-    mib[0] = CTL_KERN;
-    mib[1] = KERN_PROC;
-    mib[2] = KERN_PROC_PID;
-    mib[3] = pid;
-    len = sizeof(kp);
-    if (sysctl(mib, 4, &kp, &len, NULL, 0) == -1) {
-        return PyErr_SetFromErrno(PyExc_OSError);
+    if (! PyArg_ParseTuple(args, "l", &pid)) {
+        return NULL;
     }
-
-    if (len > 0) {
-        return Py_BuildValue("d", TV2DOUBLE(kp.ki_start));
+    if (get_kinfo_proc(pid, &kp) == -1) {
+        return NULL;
     }
-
-    return PyErr_Format(PyExc_RuntimeError, "Unable to read process start time.");
+    return Py_BuildValue("d", TV2DOUBLE(kp.ki_start));
 }
 
 
+/*
+ * Return a Python float indicating the process create time expressed in
+ * seconds since the epoch.
+ */
+static PyObject*
+get_process_io_counters(PyObject* self, PyObject* args)
+{
+    long pid;
+    struct kinfo_proc kp;
+    if (! PyArg_ParseTuple(args, "l", &pid)) {
+        return NULL;
+    }
+    if (get_kinfo_proc(pid, &kp) == -1) {
+        return NULL;
+    }
+    // there's apparently no way to determine bytes count, hence return -1.
+    return Py_BuildValue("(llll)", kp.ki_rusage.ru_inblock,
+                                   kp.ki_rusage.ru_oublock,
+                                   -1, -1);
+}
+
+
+
 /*
  * Return the RSS and VMS as a Python tuple.
  */
-static PyObject* get_memory_info(PyObject* self, PyObject* args)
+static PyObject*
+get_memory_info(PyObject* self, PyObject* args)
 {
-    int mib[4];
     long pid;
-    size_t len;
     struct kinfo_proc kp;
-
-    // the argument passed should be a process id
-	if (! PyArg_ParseTuple(args, "l", &pid)) {
-		return PyErr_Format(PyExc_RuntimeError,
-                            "Invalid argument - no PID provided.");
-	}
-
-    len = 4;
-    mib[0] = CTL_KERN;
-    mib[1] = KERN_PROC;
-    mib[2] = KERN_PROC_PID;
-    mib[3] = pid;
-    len = sizeof(kp);
-    if (sysctl(mib, 4, &kp, &len, NULL, 0) == -1) {
-        return PyErr_SetFromErrno(PyExc_OSError);
+    if (! PyArg_ParseTuple(args, "l", &pid)) {
+        return NULL;
     }
-
-    if (len > 0) {
-        return Py_BuildValue("(ll)", ptoa(kp.ki_rssize), (long)kp.ki_size);
+    if (get_kinfo_proc(pid, &kp) == -1) {
+        return NULL;
     }
-
-    return PyErr_Format(PyExc_RuntimeError, "Unable to read process start time.");
+    return Py_BuildValue("(ll)", ptoa(kp.ki_rssize), (long)kp.ki_size);
 }
 
 
@@ -393,7 +472,8 @@ static PyObject* get_memory_info(PyObject* self, PyObject* args)
  * Return a Python integer indicating the total amount of physical memory
  * in bytes.
  */
-static PyObject* get_total_phymem(PyObject* self, PyObject* args)
+static PyObject*
+get_total_phymem(PyObject* self, PyObject* args)
 {
     long total_phymem;
     int mib[2];
@@ -416,7 +496,8 @@ static PyObject* get_total_phymem(PyObject* self, PyObject* args)
  * Return a Python long indicating the amount of available physical memory in
  * bytes.
  */
-static PyObject* get_avail_phymem(PyObject* self, PyObject* args)
+static PyObject*
+get_avail_phymem(PyObject* self, PyObject* args)
 {
     unsigned long v_inactive_count = 0;
     unsigned long v_cache_count = 0;
@@ -461,7 +542,8 @@ static PyObject* get_avail_phymem(PyObject* self, PyObject* args)
  * Return a Python long indicating the total amount of virtual memory
  * in bytes.
  */
-static PyObject* get_total_virtmem(PyObject* self, PyObject* args)
+static PyObject*
+get_total_virtmem(PyObject* self, PyObject* args)
 {
     int mib[2];
     struct vmtotal vm;
@@ -485,7 +567,8 @@ static PyObject* get_total_virtmem(PyObject* self, PyObject* args)
  * Return a Python long indicating the avail amount of virtual memory
  * in bytes.
  */
-static PyObject* get_avail_virtmem(PyObject* self, PyObject* args)
+static PyObject*
+get_avail_virtmem(PyObject* self, PyObject* args)
 {
     int mib[2];
     struct vmtotal vm;
@@ -510,7 +593,8 @@ static PyObject* get_avail_virtmem(PyObject* self, PyObject* args)
 /*
  * Return a Python tuple representing user, kernel and idle CPU times
  */
-static PyObject* get_system_cpu_times(PyObject* self, PyObject* args)
+static PyObject*
+get_system_cpu_times(PyObject* self, PyObject* args)
 {
     long cpu_time[CPUSTATES];
     size_t size;
@@ -539,3 +623,142 @@ static PyObject* get_system_cpu_times(PyObject* self, PyObject* args)
                          (double)cpu_time[CP_INTR] / CLOCKS_PER_SEC
     );
 }
+
+
+/*
+ * define the psutil C module methods and initialize the module.
+ */
+static PyMethodDef
+PsutilMethods[] =
+{
+     // --- per-process functions
+
+     {"get_process_name", get_process_name, METH_VARARGS,
+        "Return process name"},
+     {"get_process_exe", get_process_exe, METH_VARARGS,
+        "Return process pathname executable"},
+     {"get_process_cmdline", get_process_cmdline, METH_VARARGS,
+        "Return process cmdline as a list of cmdline arguments"},
+     {"get_process_ppid", get_process_ppid, METH_VARARGS,
+        "Return process ppid as an integer"},
+     {"get_process_uids", get_process_uids, METH_VARARGS,
+        "Return process real effective and saved user ids as a Python tuple"},
+     {"get_process_gids", get_process_gids, METH_VARARGS,
+        "Return process real effective and saved group ids as a Python tuple"},
+     {"get_cpu_times", get_cpu_times, METH_VARARGS,
+           "Return tuple of user/kern time for the given PID"},
+     {"get_process_create_time", get_process_create_time, METH_VARARGS,
+         "Return a float indicating the process create time expressed in "
+         "seconds since the epoch"},
+     {"get_memory_info", get_memory_info, METH_VARARGS,
+         "Return a tuple of RSS/VMS memory information"},
+     {"get_process_num_threads", get_process_num_threads, METH_VARARGS,
+         "Return number of threads used by process"},
+     {"get_process_threads", get_process_threads, METH_VARARGS,
+         "Return process threads"},
+     {"get_process_status", get_process_status, METH_VARARGS,
+         "Return process status as an integer"},
+     {"get_process_io_counters", get_process_io_counters, METH_VARARGS,
+         "Return process IO counters"},
+
+
+     // --- system-related functions
+
+     {"get_pid_list", get_pid_list, METH_VARARGS,
+         "Returns a list of PIDs currently running on the system"},
+     {"get_num_cpus", get_num_cpus, METH_VARARGS,
+           "Return number of CPUs on the system"},
+     {"get_total_phymem", get_total_phymem, METH_VARARGS,
+         "Return the total amount of physical memory, in bytes"},
+     {"get_avail_phymem", get_avail_phymem, METH_VARARGS,
+         "Return the amount of available physical memory, in bytes"},
+     {"get_total_virtmem", get_total_virtmem, METH_VARARGS,
+         "Return the total amount of virtual memory, in bytes"},
+     {"get_avail_virtmem", get_avail_virtmem, METH_VARARGS,
+         "Return the amount of available virtual memory, in bytes"},
+     {"get_system_cpu_times", get_system_cpu_times, METH_VARARGS,
+         "Return system cpu times as a tuple (user, system, nice, idle, irc)"},
+     {"get_system_boot_time", get_system_boot_time, METH_VARARGS,
+         "Return a float indicating the system boot time expressed in "
+         "seconds since the epoch"},
+
+     {NULL, NULL, 0, NULL}
+};
+
+struct module_state {
+    PyObject *error;
+};
+
+#if PY_MAJOR_VERSION >= 3
+#define GETSTATE(m) ((struct module_state*)PyModule_GetState(m))
+#else
+#define GETSTATE(m) (&_state)
+static struct module_state _state;
+#endif
+
+#if PY_MAJOR_VERSION >= 3
+
+static int
+psutil_bsd_traverse(PyObject *m, visitproc visit, void *arg) {
+    Py_VISIT(GETSTATE(m)->error);
+    return 0;
+}
+
+static int
+psutil_bsd_clear(PyObject *m) {
+    Py_CLEAR(GETSTATE(m)->error);
+    return 0;
+}
+
+static struct PyModuleDef
+moduledef = {
+        PyModuleDef_HEAD_INIT,
+        "psutil_bsd",
+        NULL,
+        sizeof(struct module_state),
+        PsutilMethods,
+        NULL,
+        psutil_bsd_traverse,
+        psutil_bsd_clear,
+        NULL
+};
+
+#define INITERROR return NULL
+
+PyObject *
+PyInit__psutil_bsd(void)
+
+#else
+#define INITERROR return
+
+void init_psutil_bsd(void)
+#endif
+{
+#if PY_MAJOR_VERSION >= 3
+    PyObject *module = PyModule_Create(&moduledef);
+#else
+    PyObject *module = Py_InitModule("_psutil_bsd", PsutilMethods);
+#endif
+    PyModule_AddIntConstant(module, "SSTOP", SSTOP);
+    PyModule_AddIntConstant(module, "SSLEEP", SSLEEP);
+    PyModule_AddIntConstant(module, "SRUN", SRUN);
+    PyModule_AddIntConstant(module, "SIDL", SIDL);
+    PyModule_AddIntConstant(module, "SWAIT", SWAIT);
+    PyModule_AddIntConstant(module, "SLOCK", SLOCK);
+    PyModule_AddIntConstant(module, "SZOMB", SZOMB);
+
+    if (module == NULL) {
+        INITERROR;
+    }
+    struct module_state *st = GETSTATE(module);
+
+    st->error = PyErr_NewException("_psutil_bsd.Error", NULL, NULL);
+    if (st->error == NULL) {
+        Py_DECREF(module);
+        INITERROR;
+    }
+#if PY_MAJOR_VERSION >= 3
+    return module;
+#endif
+}
+
