@@ -1,5 +1,5 @@
 /*
- * $Id: process_info.c 523 2010-02-27 17:15:23Z jloden $
+ * $Id: process_info.c 828 2010-11-20 20:52:07Z g.rodola $
  *
  * Helper functions related to fetching process information. Used by _psutil_bsd
  * module methods.
@@ -13,6 +13,7 @@
 #include <string.h>
 #include <sys/types.h>
 #include <sys/sysctl.h>
+#include <sys/param.h>
 #include <sys/user.h>
 #include <sys/proc.h>
 
@@ -27,7 +28,8 @@
  * On success, the function returns 0.
  * On error, the function returns a BSD errno value.
  */
-int get_proc_list(struct kinfo_proc **procList, size_t *procCount)
+int
+get_proc_list(struct kinfo_proc **procList, size_t *procCount)
 {
     int err;
     struct kinfo_proc * result;
@@ -60,34 +62,30 @@ int get_proc_list(struct kinfo_proc **procList, size_t *procCount)
         assert(result == NULL);
         // Call sysctl with a NULL buffer.
         length = 0;
-        err = sysctl( (int *) name, (sizeof(name) / sizeof(*name)) - 1,
-                      NULL, &length,
-                      NULL, 0);
-        if (err == -1) {
+        err = sysctl((int *)name, (sizeof(name) / sizeof(*name)) - 1,
+                     NULL, &length, NULL, 0);
+        if (err == -1)
             err = errno;
-        }
 
         // Allocate an appropriately sized buffer based on the results
         // from the previous call.
         if (err == 0) {
             result = malloc(length);
-            if (result == NULL) {
+            if (result == NULL)
                 err = ENOMEM;
-            }
         }
 
         // Call sysctl again with the new buffer.  If we get an ENOMEM
         // error, toss away our buffer and start again.
         if (err == 0) {
-            err = sysctl( (int *) name, (sizeof(name) / sizeof(*name)) - 1,
-                          result, &length,
-                          NULL, 0);
-            if (err == -1) {
+            err = sysctl((int *) name, (sizeof(name) / sizeof(*name)) - 1,
+                          result, &length, NULL, 0);
+            if (err == -1)
                 err = errno;
-            }
             if (err == 0) {
                 done = 1;
-            } else if (err == ENOMEM) {
+            }
+            else if (err == ENOMEM) {
                 assert(result != NULL);
                 free(result);
                 result = NULL;
@@ -105,13 +103,13 @@ int get_proc_list(struct kinfo_proc **procList, size_t *procCount)
     *procList = result;
     *procCount = length / sizeof(struct kinfo_proc);
 
-    assert( (err == 0) == (*procList != NULL) );
-
+    assert((err == 0) == (*procList != NULL));
     return err;
 }
 
 
-char *getcmdpath(long pid, size_t *pathsize)
+char
+*getcmdpath(long pid, size_t *pathsize)
 {
     int  mib[4];
     char *path;
@@ -127,19 +125,15 @@ char *getcmdpath(long pid, size_t *pathsize)
 
     // call with a null buffer first to determine if we need a buffer
     if (sysctl(mib, 4, NULL, &size, NULL, 0) == -1) {
-        //perror("sysctl");
         return NULL;
     }
 
     path = malloc(size);
-    if (path == NULL) {
-        //perror("sysctl/malloc");
+    if (path == NULL)
         return NULL;
-    }
 
     *pathsize = size;
     if (sysctl(mib, 4, path, &size, NULL, 0) == -1) {
-        //perror("sysctl");
         free(path);
         return NULL;       /* Insufficient privileges */
     }
@@ -161,7 +155,8 @@ char *getcmdpath(long pid, size_t *pathsize)
  *      -1 for failure (Exception raised);
  *      1 for insufficient privileges.
  */
-char *getcmdargs(long pid, size_t *argsize)
+char
+*getcmdargs(long pid, size_t *argsize)
 {
     int mib[4];
     size_t size, argmax;
@@ -172,17 +167,13 @@ char *getcmdargs(long pid, size_t *argsize)
     mib[1] = KERN_ARGMAX;
 
     size = sizeof(argmax);
-    if (sysctl(mib, 2, &argmax, &size, NULL, 0) == -1) {
-        //perror("sysctl");
+    if (sysctl(mib, 2, &argmax, &size, NULL, 0) == -1)
         return NULL;
-    }
 
     /* Allocate space for the arguments. */
     procargs = (char *)malloc(argmax);
-    if (procargs == NULL) {
-        //printf("getcmdargs(): Cannot allocate memory for procargs\n");
+    if (procargs == NULL)
         return NULL;
-    }
 
     /*
      * Make a sysctl() call to get the raw argument space of the process.
@@ -194,8 +185,6 @@ char *getcmdargs(long pid, size_t *argsize)
 
     size = argmax;
     if (sysctl(mib, 4, procargs, &size, NULL, 0) == -1) {
-        //printf("sysctl(%lu): %s\n", pid, strerror(errno));
-        //perror("sysctl (raw args) for PID ");
         free(procargs);
         return NULL;       /* Insufficient privileges */
     }
@@ -207,7 +196,8 @@ char *getcmdargs(long pid, size_t *argsize)
 
 
 /* returns the command line as a python list object */
-PyObject* get_arg_list(long pid)
+PyObject*
+get_arg_list(long pid)
 {
     char *argstr = NULL;
     int pos = 0;
@@ -219,7 +209,7 @@ PyObject* get_arg_list(long pid)
         return retlist;
     }
 
-    // this leaks memory (grrr)
+    // XXX - this leaks memory (grrr)
     argstr = getcmdargs(pid, &argsize);
 
     if (NULL == argstr) {
@@ -251,3 +241,26 @@ PyObject* get_arg_list(long pid)
     free(argstr);
     return retlist;
 }
+
+
+/*
+ * Return 1 if PID exists in the current process list, else 0.
+ */
+int
+pid_exists(long pid)
+{
+    int kill_ret;
+    if (pid < 0) {
+        return 0;
+    }
+
+    // if kill returns success of permission denied we know it's a valid PID
+    kill_ret = kill(pid , 0);
+    if ((0 == kill_ret) || (EPERM == errno)) {
+        return 1;
+    }
+
+    // otherwise return 0 for PID not found
+    return 0;
+}
+
