@@ -1,8 +1,8 @@
 /*
- * $Id: process_info.c 521 2010-02-27 16:19:10Z billiejoex $
+ * $Id: process_info.c 931 2011-02-22 04:10:46Z g.rodola $
  *
- * Helper functions related to fetching process information. Used by _psutil_mswindows
- * module methods.
+ * Helper functions related to fetching process information. Used by
+ * _psutil_mswindows module methods.
  */
 
 #include <Python.h>
@@ -12,13 +12,11 @@
 
 #include "security.h"
 #include "process_info.h"
-
+#include "ntextapi.h"
 
 /*
  * NtQueryInformationProcess code taken from
- *
  * http://wj32.wordpress.com/2009/01/24/howto-get-the-command-line-of-processes/
- *
  * typedefs needed to compile against ntdll functions not exposted in the API
  */
 typedef LONG NTSTATUS;
@@ -31,26 +29,66 @@ typedef NTSTATUS (NTAPI *_NtQueryInformationProcess)(
     PDWORD ReturnLength
     );
 
-typedef struct _UNICODE_STRING
-{
-    USHORT Length;
-    USHORT MaximumLength;
-    PWSTR Buffer;
-} UNICODE_STRING, *PUNICODE_STRING;
-
 typedef struct _PROCESS_BASIC_INFORMATION
 {
-    DWORD ExitStatus;
+    PVOID Reserved1;
     PVOID PebBaseAddress;
-    DWORD AffinityMask;
-    DWORD BasePriority;
-    DWORD UniqueProcessId;
-    DWORD ParentProcessId;
+    PVOID Reserved2[2];
+    ULONG_PTR UniqueProcessId;
+    PVOID Reserved3;
 } PROCESS_BASIC_INFORMATION, *PPROCESS_BASIC_INFORMATION;
 
 
+/*
+ * A wrapper around OpenProcess setting NSP exception if process
+ * no longer exists.
+ * "pid" is the process pid, "dwDesiredAccess" is the first argument
+ * exptected by OpenProcess.
+ * Return a process handle or NULL.
+ */
+HANDLE
+handle_from_pid_waccess(DWORD pid, DWORD dwDesiredAccess)
+{
+    HANDLE hProcess;
+    DWORD  processExitCode = 0;
+
+    hProcess = OpenProcess(dwDesiredAccess, FALSE, pid);
+    if (hProcess == NULL) {
+        if (GetLastError() == ERROR_INVALID_PARAMETER) {
+            NoSuchProcess();
+        }
+        else {
+            PyErr_SetFromWindowsErr(0);
+        }
+        return NULL;
+    }
+
+    /* make sure the process is running */
+    GetExitCodeProcess(hProcess, &processExitCode);
+    if (processExitCode == 0) {
+        NoSuchProcess();
+        CloseHandle(hProcess);
+        return NULL;
+    }
+    return hProcess;
+}
+
+
+/*
+ * Same as handle_from_pid_waccess but implicitly uses
+ * PROCESS_QUERY_INFORMATION | PROCESS_VM_READ as dwDesiredAccess
+ * parameter for OpenProcess.
+ */
+HANDLE
+handle_from_pid(DWORD pid) {
+    DWORD dwDesiredAccess = PROCESS_QUERY_INFORMATION | PROCESS_VM_READ;
+    return handle_from_pid_waccess(pid, dwDesiredAccess);
+}
+
+
 // fetch the PEB base address from NtQueryInformationProcess()
-PVOID GetPebAddress(HANDLE ProcessHandle)
+PVOID
+GetPebAddress(HANDLE ProcessHandle)
 {
     _NtQueryInformationProcess NtQueryInformationProcess =
         (_NtQueryInformationProcess)GetProcAddress(
@@ -62,53 +100,50 @@ PVOID GetPebAddress(HANDLE ProcessHandle)
 }
 
 
-DWORD* get_pids(DWORD *numberOfReturnedPIDs){
-	int procArraySz = 1024;
+DWORD*
+get_pids(DWORD *numberOfReturnedPIDs) {
+    int procArraySz = 1024;
 
-	/* Win32 SDK says the only way to know if our process array
-	 * wasn't large enough is to check the returned size and make
-	 * sure that it doesn't match the size of the array.
-	 * If it does we allocate a larger array and try again*/
+    /* Win32 SDK says the only way to know if our process array
+     * wasn't large enough is to check the returned size and make
+     * sure that it doesn't match the size of the array.
+     * If it does we allocate a larger array and try again*/
 
-	/* Stores the actual array */
-	DWORD *procArray = NULL;
-	DWORD procArrayByteSz;
+    /* Stores the actual array */
+    DWORD *procArray = NULL;
+    DWORD procArrayByteSz;
 
     /* Stores the byte size of the returned array from enumprocesses */
-	DWORD enumReturnSz = 0;
-
-	do {
-		free(procArray);
+    DWORD enumReturnSz = 0;
 
+    do {
+        free(procArray);
         procArrayByteSz = procArraySz * sizeof(DWORD);
-		procArray = malloc(procArrayByteSz);
+        procArray = malloc(procArrayByteSz);
 
+        if (! EnumProcesses(procArray, procArrayByteSz, &enumReturnSz)) {
+            free(procArray);
+            PyErr_SetFromWindowsErr(0);
+            return NULL;
+        }
+        else if (enumReturnSz == procArrayByteSz) {
+            /* Process list was too large.  Allocate more space*/
+            procArraySz += 1024;
+        }
 
-		if (! EnumProcesses(procArray, procArrayByteSz, &enumReturnSz))
-		{
-			free(procArray);
-			/* Throw exception to python */
-			PyErr_SetString(PyExc_RuntimeError, "EnumProcesses failed");
-			return NULL;
-		}
-		else if (enumReturnSz == procArrayByteSz)
-		{
-			/* Process list was too large.  Allocate more space*/
-			procArraySz += 1024;
-		}
+        /* else we have a good list */
 
-		/* else we have a good list */
+    } while(enumReturnSz == procArraySz * sizeof(DWORD));
 
-	} while(enumReturnSz == procArraySz * sizeof(DWORD));
-
-	/* The number of elements is the returned size / size of each element */
+    /* The number of elements is the returned size / size of each element */
     *numberOfReturnedPIDs = enumReturnSz / sizeof(DWORD);
 
     return procArray;
 }
 
 
-int is_system_proc(DWORD pid) {
+int
+is_system_proc(DWORD pid) {
     HANDLE hProcess;
 
     // Special case for PID 0 System Idle Process
@@ -116,12 +151,11 @@ int is_system_proc(DWORD pid) {
     if ((pid == 0) || (pid == 4)) {
         return 1;
     }
-
     if (pid < 0) {
         return 0;
     }
 
-    hProcess = handle_from_pid(pid);
+    hProcess = OpenProcess(PROCESS_QUERY_INFORMATION | PROCESS_VM_READ, FALSE, pid);
     if (NULL == hProcess) {
         // invalid parameter is no such process
         if (GetLastError() == ERROR_INVALID_PARAMETER) {
@@ -141,7 +175,8 @@ int is_system_proc(DWORD pid) {
 }
 
 
-int pid_is_running(DWORD pid)
+int
+pid_is_running(DWORD pid)
 {
     HANDLE hProcess;
     DWORD exitCode;
@@ -155,37 +190,45 @@ int pid_is_running(DWORD pid)
         return 0;
     }
 
-    hProcess = handle_from_pid(pid);
+    hProcess = OpenProcess(PROCESS_QUERY_INFORMATION | PROCESS_VM_READ,
+                           FALSE, pid);
     if (NULL == hProcess) {
         // invalid parameter is no such process
         if (GetLastError() == ERROR_INVALID_PARAMETER) {
+            CloseHandle(hProcess);
             return 0;
         }
 
         // access denied obviously means there's a process to deny access to...
         if (GetLastError() == ERROR_ACCESS_DENIED) {
+            CloseHandle(hProcess);
             return 1;
         }
 
+        CloseHandle(hProcess);
         PyErr_SetFromWindowsErr(0);
         return -1;
     }
 
     if (GetExitCodeProcess(hProcess, &exitCode)) {
+        CloseHandle(hProcess);
         return (exitCode == STILL_ACTIVE);
     }
 
     // access denied means there's a process there so we'll assume it's running
     if (GetLastError() == ERROR_ACCESS_DENIED) {
+        CloseHandle(hProcess);
         return 1;
     }
 
     PyErr_SetFromWindowsErr(0);
+    CloseHandle(hProcess);
     return -1;
 }
 
 
-int pid_in_proclist(DWORD pid)
+int
+pid_in_proclist(DWORD pid)
 {
     DWORD *proclist = NULL;
     DWORD numberOfReturnedPIDs;
@@ -193,8 +236,6 @@ int pid_in_proclist(DWORD pid)
 
     proclist = get_pids(&numberOfReturnedPIDs);
     if (NULL == proclist) {
-		PyErr_SetString(PyExc_RuntimeError,
-                        "get_pids() failed for pid_in_proclist()");
         return -1;
     }
 
@@ -210,20 +251,6 @@ int pid_in_proclist(DWORD pid)
 }
 
 
-// Get a process handle from a pid with PROCESS_QUERY_INFORMATION rights
-HANDLE handle_from_pid(DWORD pid)
-{
-    HANDLE hProcess = OpenProcess(PROCESS_QUERY_INFORMATION | PROCESS_VM_READ,
-                                  FALSE, pid);
-    /*
-    if (NULL == hProcess) {
-        hProcess = OpenProcess(PROCESS_QUERY_INFORMATION, FALSE, pid);
-    }
-    */
-    return hProcess;
-}
-
-
 // Check exit code from a process handle. Return FALSE on an error also
 BOOL is_running(HANDLE hProcess)
 {
@@ -242,57 +269,57 @@ BOOL is_running(HANDLE hProcess)
 
 // Return None to represent NoSuchProcess, else return NULL for
 // other exception or the name as a Python string
-PyObject* get_name(long pid)
+PyObject*
+get_name(long pid)
 {
-	HANDLE h = CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS, 0);
-	PROCESSENTRY32 pe = { 0 };
-	pe.dwSize = sizeof(PROCESSENTRY32);
+    HANDLE h = CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS, 0);
+    PROCESSENTRY32 pe = { 0 };
+    pe.dwSize = sizeof(PROCESSENTRY32);
 
-	if( Process32First(h, &pe)) {
-	    do {
-			if (pe.th32ProcessID == pid) {
+    if( Process32First(h, &pe)) {
+        do {
+            if (pe.th32ProcessID == pid) {
                 CloseHandle(h);
                 return Py_BuildValue("s", pe.szExeFile);
-			}
-		} while( Process32Next(h, &pe));
+            }
+        } while(Process32Next(h, &pe));
 
-        // the process was never found (NoSuchProcess raised later)
+        // the process was never found, set NoSuchProcess exception
+        NoSuchProcess();
         CloseHandle(h);
-        return Py_BuildValue("");
-	}
+        return NULL;
+    }
 
     CloseHandle(h);
-    PyErr_SetString(PyExc_RuntimeError,
-                    "Failed to read process name from toolhelp snapshot");
-    return NULL;
+    return PyErr_SetFromWindowsErr(0);
 }
 
 
 /* returns parent pid (as a Python int) for given pid or None on failure */
-PyObject* get_ppid(long pid)
+PyObject*
+get_ppid(long pid)
 {
-	HANDLE h = CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS, 0);
-	PROCESSENTRY32 pe = { 0 };
-	pe.dwSize = sizeof(PROCESSENTRY32);
+    HANDLE h = CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS, 0);
+    PROCESSENTRY32 pe = { 0 };
+    pe.dwSize = sizeof(PROCESSENTRY32);
 
-	if( Process32First(h, &pe)) {
-	    do {
-			if (pe.th32ProcessID == pid) {
-				//printf("PID: %i; PPID: %i\n", pid, pe.th32ParentProcessID);
+    if( Process32First(h, &pe)) {
+        do {
+            if (pe.th32ProcessID == pid) {
+                //printf("PID: %i; PPID: %i\n", pid, pe.th32ParentProcessID);
                 CloseHandle(h);
-                return Py_BuildValue("l", pe.th32ParentProcessID);
-			}
-		} while( Process32Next(h, &pe));
+                return Py_BuildValue("I", pe.th32ParentProcessID);
+            }
+        } while(Process32Next(h, &pe));
 
-        // the process was never found (NoSuchProcess raised later)
+        // the process was never found, set NoSuchProcess exception
+        NoSuchProcess();
         CloseHandle(h);
-        return Py_BuildValue("");
-	}
+        return NULL;
+    }
 
     CloseHandle(h);
-    PyErr_SetString(PyExc_RuntimeError,
-                    "Failed to read process ppid from toolhelp snapshot");
-    return NULL;
+    return PyErr_SetFromWindowsErr(0);
 }
 
 
@@ -301,7 +328,8 @@ PyObject* get_ppid(long pid)
  * returns a Python list representing the arguments for the process
  * with given pid or NULL on error.
  */
-PyObject* get_arg_list(long pid)
+PyObject*
+get_arg_list(long pid)
 {
     int nArgs, i;
     LPWSTR *szArglist;
@@ -316,29 +344,40 @@ PyObject* get_arg_list(long pid)
 
 
     hProcess = handle_from_pid(pid);
-    if(NULL == hProcess) {
-        //printf("Could not open process!: %lu\n", GetLastError());
-        return PyErr_SetFromWindowsErr(0);
+    if(hProcess == NULL) {
+        return NULL;
     }
 
     pebAddress = GetPebAddress(hProcess);
 
     /* get the address of ProcessParameters */
+#ifdef _WIN64
+    if (!ReadProcessMemory(hProcess, (PCHAR)pebAddress + 32,
+        &rtlUserProcParamsAddress, sizeof(PVOID), NULL))
+#else
     if (!ReadProcessMemory(hProcess, (PCHAR)pebAddress + 0x10,
         &rtlUserProcParamsAddress, sizeof(PVOID), NULL))
+#endif
     {
         //printf("Could not read the address of ProcessParameters!\n");
+        PyErr_SetFromWindowsErr(0);
         CloseHandle(hProcess);
-        return PyErr_SetFromWindowsErr(0);
+        return NULL;
     }
 
     /* read the CommandLine UNICODE_STRING structure */
+#ifdef _WIN64
+    if (!ReadProcessMemory(hProcess, (PCHAR)rtlUserProcParamsAddress + 112,
+        &commandLine, sizeof(commandLine), NULL))
+#else
     if (!ReadProcessMemory(hProcess, (PCHAR)rtlUserProcParamsAddress + 0x40,
         &commandLine, sizeof(commandLine), NULL))
+#endif
     {
         //printf("Could not read CommandLine!\n");
+        PyErr_SetFromWindowsErr(0);
         CloseHandle(hProcess);
-        return PyErr_SetFromWindowsErr(0);
+        return NULL;
     }
 
 
@@ -350,9 +389,10 @@ PyObject* get_arg_list(long pid)
         commandLineContents, commandLine.Length, NULL))
     {
         //printf("Could not read the command line string!\n");
+        PyErr_SetFromWindowsErr(0);
         CloseHandle(hProcess);
         free(commandLineContents);
-        return PyErr_SetFromWindowsErr(0);
+        return NULL;
     }
 
     /* print the commandline */
@@ -380,7 +420,7 @@ PyObject* get_arg_list(long pid)
         // arglist parsed as array of UNICODE_STRING, so convert each to Python
         // string object and add to arg list
         argList = Py_BuildValue("[]");
-        for( i=0; i<nArgs; i++) {
+        for(i=0; i<nArgs; i++) {
             //printf("%d: %.*S (%d characters)\n", i, wcslen(szArglist[i]),
             //                  szArglist[i], wcslen(szArglist[i]));
             arg_from_wchar = PyUnicode_FromWideChar(szArglist[i],
@@ -403,3 +443,80 @@ PyObject* get_arg_list(long pid)
     return argList;
 }
 
+
+#define PH_FIRST_PROCESS(Processes) ((PSYSTEM_PROCESS_INFORMATION)(Processes))
+
+#define PH_NEXT_PROCESS(Process) ( \
+    ((PSYSTEM_PROCESS_INFORMATION)(Process))->NextEntryOffset ? \
+    (PSYSTEM_PROCESS_INFORMATION)((PCHAR)(Process) + \
+    ((PSYSTEM_PROCESS_INFORMATION)(Process))->NextEntryOffset) : \
+    NULL \
+    )
+
+const STATUS_INFO_LENGTH_MISMATCH = 0xC0000004;
+const STATUS_BUFFER_TOO_SMALL = 0xC0000023L;
+
+/*
+ * Given a process PID and a PSYSTEM_PROCESS_INFORMATION structure
+ * fills the structure with process information.
+ * On success return 1, else 0 with Python exception already set.
+ */
+int
+get_process_info(DWORD pid, PSYSTEM_PROCESS_INFORMATION *retProcess)
+{
+    static ULONG initialBufferSize = 0x4000;
+    NTSTATUS status;
+    PVOID buffer;
+    ULONG bufferSize;
+    PSYSTEM_PROCESS_INFORMATION process;
+
+    // get NtQuerySystemInformation
+    typedef DWORD (_stdcall *NTQSI_PROC) (int, PVOID, ULONG, PULONG);
+    NTQSI_PROC NtQuerySystemInformation;
+    HINSTANCE hNtDll;
+    hNtDll = LoadLibrary(TEXT("ntdll.dll"));
+    NtQuerySystemInformation = (NTQSI_PROC)GetProcAddress(
+                                hNtDll, "NtQuerySystemInformation");
+
+    bufferSize = initialBufferSize;
+    buffer = malloc(bufferSize);
+
+    while (TRUE) {
+        status = NtQuerySystemInformation(SystemProcessInformation, buffer,
+                                          bufferSize, &bufferSize);
+
+        if (status == STATUS_BUFFER_TOO_SMALL || status == STATUS_INFO_LENGTH_MISMATCH)
+        {
+            free(buffer);
+            buffer = malloc(bufferSize);
+        }
+        else {
+            break;
+        }
+    }
+
+    if (status != 0) {
+        free(buffer);
+        PyErr_Format(PyExc_RuntimeError, "NtQuerySystemInformation() failed");
+        return 0;
+    }
+
+
+    if (bufferSize <= 0x20000)
+        initialBufferSize = bufferSize;
+
+    process = PH_FIRST_PROCESS(buffer);
+    do {
+        if (process->UniqueProcessId == (HANDLE)pid) {
+            free(buffer);
+            *retProcess = process;
+            return 1;
+        }
+    } while (process = PH_NEXT_PROCESS(process));
+
+    free(buffer);
+    NoSuchProcess();
+    return 0;
+}
+
+
