@@ -1,5 +1,5 @@
 /*
- * $Id: process_info.h 435 2009-09-19 09:32:29Z jloden $
+ * $Id: process_info.h 930 2011-02-22 03:45:38Z g.rodola $
  *
  * Helper functions related to fetching process information. Used by _psutil_mswindows
  * module methods.
@@ -8,7 +8,10 @@
 #include <Python.h>
 #include <windows.h>
 
-
+PyObject * NoSuchProcess(void);
+PyObject * AccessDenied(void);
+HANDLE handle_from_pid_waccess(DWORD pid, DWORD dwDesiredAccess);
+HANDLE handle_from_pid(DWORD pid);
 PVOID GetPebAddress(HANDLE ProcessHandle);
 HANDLE handle_from_pid(DWORD pid);
 BOOL is_running(HANDLE hProcess);
@@ -19,3 +22,4 @@ PyObject* get_arg_list(long pid);
 PyObject* get_ppid(long pid);
 PyObject* get_name(long pid);
 DWORD* get_pids(DWORD *numberOfReturnedPIDs);
+
