@@ -1,5 +1,5 @@
 /*
- * $Id: security.h 247 2009-03-12 18:47:35Z jloden $
+ * $Id: security.h 435 2009-09-19 09:32:29Z jloden $
  *
  * Security related functions for Windows platform (Set privileges such as
  * SeDebug), as well as security helper functions.
@@ -11,4 +11,6 @@
 BOOL SetPrivilege(HANDLE hToken, LPCTSTR Privilege, BOOL bEnablePrivilege);
 int SetSeDebug();
 int UnsetSeDebug();
+HANDLE token_from_handle(HANDLE hProcess);
+int HasSystemPrivilege(HANDLE hProcess);
 
