@@ -1,5 +1,5 @@
 /*
- * $Id: process_info.h 399 2009-05-12 23:24:21Z jloden $
+ * $Id: process_info.h 823 2010-11-16 22:34:06Z jloden $
  *
  * Helper functions related to fetching process information. Used by _psutil_osx
  * module methods.
@@ -11,6 +11,7 @@
 typedef struct kinfo_proc kinfo_proc;
 
 int get_proc_list(kinfo_proc **procList, size_t *procCount);
-int getcmdargs(long pid, PyObject **exec_path, PyObject **envlist, PyObject **arglist);
+int get_kinfo_proc(pid_t pid, struct kinfo_proc *kp);
+size_t get_argmax(void);
+int pid_exists(long pid);
 PyObject* get_arg_list(long pid);
-
