@@ -1,17 +1,27 @@
 #!/usr/bin/env python
 #
-# $Id: test_memory_leaks.py 510 2010-02-22 21:43:18Z billiejoex $
+# $Id: test_memory_leaks.py 828 2010-11-20 20:52:07Z g.rodola $
 #
 
+"""
+Note: this is targeted for python 2.x.
+To run it under python 3.x you need to use 2to3 tool first:
+
+$ 2to3 -w test/test_memory_leaks.py
+"""
+
+
 import os
 import gc
 import sys
 import unittest
 
 import psutil
+from test_psutil import reap_children, skipUnless, skipIf, \
+                        POSIX, LINUX, WINDOWS, OSX, BSD
 
 LOOPS = 1000
-MARGIN = 4096
+TOLERANCE = 4096
 
 
 class TestProcessObjectLeaks(unittest.TestCase):
@@ -20,10 +30,13 @@ class TestProcessObjectLeaks(unittest.TestCase):
     def setUp(self):
         gc.collect()
 
+    def tearDown(self):
+        reap_children()
+
     def execute(self, method, *args, **kwarks):
         # step 1
+        p = psutil.Process(os.getpid())
         for x in xrange(LOOPS):
-            p = psutil.Process(os.getpid())
             obj = getattr(p, method)
             if callable(obj):
                 retvalue = obj(*args, **kwarks)
@@ -34,8 +47,8 @@ class TestProcessObjectLeaks(unittest.TestCase):
         rss1 = psutil.Process(os.getpid()).get_memory_info()[0]
 
         # step 2
+        p = psutil.Process(os.getpid())
         for x in xrange(LOOPS):
-            p = psutil.Process(os.getpid())
             obj = getattr(p, method)
             if callable(obj):
                 retvalue = obj(*args, **kwarks)
@@ -47,45 +60,61 @@ class TestProcessObjectLeaks(unittest.TestCase):
 
         # comparison
         difference = rss2 - rss1
-        if difference > MARGIN:
+        if difference > TOLERANCE:
             self.fail("rss1=%s, rss2=%s, difference=%s" %(rss1, rss2, difference))
 
-    def test_pid(self):
-        self.execute('pid')
+    def test_name(self):
+        self.execute('name')
 
-    def test__str__(self):
-        # includes name, ppid, path, cmdline, uid, gid properties
-        self.execute('__str__')
+    def test_cmdline(self):
+        self.execute('cmdline')
 
-    # XXX - disabled for Windows as it uses WMI which takes ages to complete
-    if not sys.platform.lower().startswith("win32"):
-        def test_username(self):
-            self.execute('username')
+    def test_ppid(self):
+        self.execute('ppid')
+
+    def test_uid(self):
+        self.execute('uid')
+
+    def test_uid(self):
+        self.execute('gid')
+
+    @skipIf(POSIX)
+    def test_username(self):
+        self.execute('username')
 
     def test_create_time(self):
         self.execute('create_time')
 
+    def test_get_num_threads(self):
+        self.execute('get_num_threads')
+
+    def test_get_threads(self):
+        self.execute('get_num_threads')
+
     def test_get_cpu_times(self):
         self.execute('get_cpu_times')
 
-    def test_get_cpu_percent(self):
-        self.execute('get_cpu_percent')
-
     def test_get_memory_info(self):
         self.execute('get_memory_info')
 
-    def test_get_memory_percent(self):
-        self.execute('get_memory_percent')
-
     def test_is_running(self):
         self.execute('is_running')
 
+    @skipUnless(WINDOWS)
     def test_resume(self):
         self.execute('resume')
 
-    if hasattr(psutil.Process, "getcwd"):
-        def test_getcwd(self):
-            self.execute('getcwd')
+    @skipUnless(WINDOWS)
+    def test_getcwd(self):
+        self.execute('getcwd')
+
+    @skipUnless(WINDOWS)
+    def test_get_open_files(self):
+        self.execute('get_open_files')
+
+    @skipUnless(WINDOWS)
+    def test_get_connections(self):
+        self.execute('get_connections')
 
 
 class TestModuleFunctionsLeaks(unittest.TestCase):
@@ -119,12 +148,13 @@ class TestModuleFunctionsLeaks(unittest.TestCase):
 
         # comparison
         difference = rss2 - rss1
-        if difference > MARGIN:
+        if difference > TOLERANCE:
             self.fail("rss1=%s, rss2=%s, difference=%s" %(rss1, rss2, difference))
 
     def test_get_pid_list(self):
         self.execute('get_pid_list')
 
+    @skipIf(POSIX)
     def test_pid_exists(self):
         self.execute('pid_exists', os.getpid())
 
@@ -149,9 +179,6 @@ class TestModuleFunctionsLeaks(unittest.TestCase):
     def test_cpu_times(self):
         self.execute('cpu_times')
 
-    def test_cpu_percent(self):
-        self.execute('cpu_percent')
-
 
 def test_main():
     test_suite = unittest.TestSuite()
